@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include <string>
 
 #include "base/file_utilities.h"
@@ -349,7 +347,7 @@ static bool parse_shortcut(const std::string &shortcut, mdc::KeyInfo &key, mdc::
   std::vector<std::string> parts= base::split(shortcut, "+");
   mods= mdc::SNone;
 
-  for (size_t c= parts.size()-1, i= 0; i < c; i++)
+  for (ssize_t c= parts.size() - 1, i = 0; i < c; i++)
   {
     std::string mod= parts[i];
     if (mod == "control")
@@ -474,7 +472,8 @@ void CommandUI::add_recent_menu(mforms::MenuItem *parent)
     std::string caption;
     if (i < 9)
     {
-      caption= strfmt("%li %s", i+1, strlist.get(i).c_str());
+      caption= strfmt("%li %s", (long)i+1, strlist.get(i).c_str());
+
 #if !defined(_WIN32) && !defined(__APPLE__)
       caption= "_"+replace_string(caption, "_", "__");
 #endif
@@ -495,8 +494,9 @@ void CommandUI::add_recent_menu(mforms::MenuItem *parent)
     }
     
     item = mforms::manage(new mforms::MenuItem(caption));
-    item->set_name(strfmt("wb.file.openRecentModel:%li", i+1));
-    scoped_connect(item->signal_clicked(),boost::bind(&WBContext::open_recent_document, _wb, i+1));
+    item->set_name(strfmt("wb.file.openRecentModel:%li", (long)i+1));
+
+    scoped_connect(item->signal_clicked(), boost::bind(&WBContext::open_recent_document, _wb, (int)i + 1));
     parent->add_item(item);
   }
 }
@@ -564,11 +564,9 @@ void CommandUI::add_scripts_menu(mforms::MenuItem *parent)
   try
   {
     std::list<std::string> pyfiles = base::scan_for_files_matching(bec::make_path(_wb->get_grt_manager()->get_user_script_path(), "*.py"));
-    std::list<std::string> luafiles = base::scan_for_files_matching(bec::make_path(_wb->get_grt_manager()->get_user_script_path(), "*.lua"));
     std::vector<std::string> files;
 
     std::copy(pyfiles.begin(), pyfiles.end(), std::back_inserter(files));
-    std::copy(luafiles.begin(), luafiles.end(), std::back_inserter(files));
     std::sort(files.begin(), files.end());
 
     for (std::vector<std::string>::const_iterator f = files.begin(); f != files.end(); ++f)
@@ -654,6 +652,7 @@ void CommandUI::add_menu_items_for_context(const std::string &context, mforms::M
           continue;
         item = mforms::manage(new mforms::MenuItem(mitem->caption(), type));
         item->set_name(mitem->name());
+        parent->add_item(item);
         update_item_state(mitem, cmd, item);
         
         add_menu_items_for_context(context, item, mitem);
@@ -664,6 +663,7 @@ void CommandUI::add_menu_items_for_context(const std::string &context, mforms::M
           continue;
         item = mforms::manage(new mforms::MenuItem("", mforms::SeparatorMenuItem));
         item->set_name(mitem->name());
+        parent->add_item(item);
       }
       else
       { 
@@ -733,8 +733,10 @@ void CommandUI::add_menu_items_for_context(const std::string &context, mforms::M
         }
         
         item = mforms::manage(new mforms::MenuItem(caption, type));
+        parent->add_item(item);
         item->set_name(cmd.args.empty() ? cmd.name : cmd.name+":"+cmd.args);
-        item->set_shortcut(mitem->shortcut());
+        if (!mitem->shortcut().empty())
+          item->set_shortcut(mitem->shortcut());
         scoped_connect(item->signal_clicked(),boost::bind(activate_slot, mitem->command()));
         //update_item_state(mitem, cmd, item);
         if (!enabled)
@@ -755,8 +757,6 @@ void CommandUI::add_menu_items_for_context(const std::string &context, mforms::M
 #endif
       
       added_menu_items.insert(mitem->name());
-      
-      parent->add_item(item);
     }
   }
 
@@ -1072,7 +1072,7 @@ void CommandUI::add_builtin_command(const std::string &name,
   cmd.validate= validate;
 
   if (_builtin_commands.find(name) != _builtin_commands.end())
-    g_message("%s built-in command is being overwritten", name.c_str());
+    log_warning("%s built-in command is being overwritten", name.c_str());
   
   _builtin_commands[name]= cmd;
 }
