@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,25 +17,28 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "grt_dispatcher.h"
 #include "grt_manager.h"
 #include <grtpp_util.h>
 #include <grtpp_shell.h>
 #include "base/log.h"
+#include "base/threading.h"
+#include "driver_manager.h"
+
+
+#include "mforms/utilities.h"
 
 using namespace bec;
 
-static bool debug_dispatcher= false;
-static GStaticRecMutex retain_mutex = G_STATIC_REC_MUTEX_INIT;
-
 DEFAULT_LOG_DOMAIN("GRTDispatcher");
 
+// Extra debug print flag for additional information (not dynamically switchable like log calls).
+static bool debug_dispatcher = false;
+
 #ifdef __GNUC__
-#define DPRINT(fmt, ...) if (debug_dispatcher) g_message(fmt,##__VA_ARGS__)
+#define DPRINT(fmt, ...) if (debug_dispatcher) log_debug3(fmt,##__VA_ARGS__)
 #else
-#define DPRINT(...) if (debug_dispatcher) g_message(__VA_ARGS__)
+#define DPRINT(...) if (debug_dispatcher) log_debug3(__VA_ARGS__)
 #endif
 
 class NULLTask : public GRTTaskBase
@@ -119,20 +122,17 @@ void GRTTaskBase::cancel()
 
 void GRTTaskBase::retain()
 {
-  g_static_rec_mutex_lock(&retain_mutex);
-  _refcount++;
-  g_static_rec_mutex_unlock(&retain_mutex);
+  base::atomic_int_inc(&_refcount);
 }
 
 
 void GRTTaskBase::release()
 {
   bool do_delete = false;
-  g_static_rec_mutex_lock(&retain_mutex);
-  --_refcount;
-  if (_refcount == 0)
+
+  if (base::atomic_int_dec_and_test_if_zero(&_refcount))
     do_delete = true; // delete can cause a side-effect with recursion, which would deadlock
-  g_static_rec_mutex_unlock(&retain_mutex);
+
   if (do_delete)
     delete this;
 }
@@ -187,26 +187,6 @@ void GRTTaskBase::failed_m(const std::exception &exc)
   release();
 }
 
-
-struct MutexCentry
-{
-  MutexCentry(GMutex *mtx) : _mtx(mtx) 
-  { 
-    g_mutex_lock(mtx);
-  }
-
-  ~MutexCentry() 
-  { 
-    g_mutex_unlock(_mtx);
-  }
-private:
-  GMutex *_mtx;
-
-  MutexCentry(MutexCentry&);
-  MutexCentry& operator = (MutexCentry&);
-};
-
-
 bool GRTTaskBase::process_message(const grt::Message &msg)
 {
   retain();
@@ -338,10 +318,9 @@ static void sleep_2ms()
 static GThread *_main_thread= 0;
 
 GRTDispatcher::GRTDispatcher(grt::GRT *grt, bool threaded, bool is_main_dispatcher)
-  : _busy(false), _threading_disabled(!threaded), _is_main_dispatcher(is_main_dispatcher), _shut_down(false), _grt(grt), _current_task(NULL)
+  : _busy(0), _threading_disabled(!threaded), _w_runing(0), _is_main_dispatcher(is_main_dispatcher), _shut_down(false), _grt(grt), _current_task(NULL)
 {
-  _worker_running= false;
-  _worker_shutdown= false;
+  _shutdown_callback= false;
 
   if (threaded)
   {
@@ -382,11 +361,12 @@ void GRTDispatcher::start(boost::shared_ptr<GRTDispatcher> self)
   _shut_down = false;
   if (!_threading_disabled)
   {
-    DPRINT("starting worker thread");
-    _thread= g_thread_create(worker_thread, this, FALSE, NULL);
+
+    log_debug("starting worker thread\n");
+    _thread= base::create_thread(worker_thread, this);
     if (_thread == 0)
     {
-      log_error("g_thread_create failed to create the GRT worker thread. Falling back into non-threaded mode.\n");
+      log_error("base::create_thread failed to create the GRT worker thread. Falling back into non-threaded mode.\n");
       _threading_disabled = true;
     }
   }
@@ -414,13 +394,14 @@ void GRTDispatcher::shutdown()
 //    _grt->pop_status_query_handler();
   }
 
-  _worker_shutdown= true;
-  if (!_threading_disabled && _worker_running)
+  _shutdown_callback= true;
+  if (!_threading_disabled && _thread != 0) // _thread == 0, means that init was not called, but threading_disabled was set to false.
   {
     NULLTask *task= new NULLTask(this);
     add_task(task);
-    DPRINT("Main thread waiting for worker to finish");
-    while (_worker_running) g_usleep(100000);
+    log_debug2("GRTDispatcher:Main thread waiting for worker to finish\n");
+    _w_runing.wait();
+    log_debug2("GRTDispatcher:Main thread worker finished\n");
   }
 
   bec::GRTManager *grtm= bec::GRTManager::get_instance_for(_grt);
@@ -428,36 +409,6 @@ void GRTDispatcher::shutdown()
     grtm->remove_dispatcher(this);
 }
 
-#ifdef _WIN32
-const DWORD MS_VC_EXCEPTION=0x406D1388;
-
-#pragma pack(push,8)
-typedef struct tagTHREADNAME_INFO
-{
-   DWORD dwType; // Must be 0x1000.
-   LPCSTR szName; // Pointer to name (in user addr space).
-   DWORD dwThreadID; // Thread ID (-1=caller thread).
-   DWORD dwFlags; // Reserved for future use, must be zero.
-} THREADNAME_INFO;
-#pragma pack(pop)
-
-void SetThreadName( DWORD dwThreadID, char* threadName)
-{
-   THREADNAME_INFO info;
-   info.dwType = 0x1000;
-   info.szName = threadName;
-   info.dwThreadID = dwThreadID;
-   info.dwFlags = 0;
-
-   __try
-   {
-      RaiseException( MS_VC_EXCEPTION, 0, sizeof(info)/sizeof(ULONG_PTR), (ULONG_PTR*)&info );
-   }
-   __except(EXCEPTION_CONTINUE_EXECUTION)
-   {
-   }
-}
-#endif
 
 gpointer GRTDispatcher::worker_thread(gpointer data)
 {
@@ -465,43 +416,44 @@ gpointer GRTDispatcher::worker_thread(gpointer data)
   GAsyncQueue *task_queue= self->_task_queue;
   GAsyncQueue *callback_queue= self->_callback_queue;
 
-#if defined(_WIN32) && defined(_DEBUG)
-  SetThreadName(-1, "WB dispatcher");
-#endif
+  mforms::Utilities::set_thread_name("GRTDispatcher");
 
-  DPRINT("worker thread running");
+  log_debug("worker thread running\n");
   
   g_async_queue_ref(task_queue);
   g_async_queue_ref(callback_queue);
 
   self->worker_thread_init();
   
-  self->_worker_running= true;
-  while (!self->_worker_shutdown)
+//  self->_worker_running= true;
+  while (true)
   {
     GRTTaskBase *task;
-    GTimeVal timeout;
 
-    self->_busy= false;
     self->worker_thread_iteration();
 
-    DPRINT("worker: waiting task...");
     // pop next task pushed to queue by the main thread
-    
+
+#if GLIB_CHECK_VERSION(2,32,0)
+    task= static_cast<GRTTaskBase*>(g_async_queue_timeout_pop(task_queue, 1000000));
+#else
+    GTimeVal timeout;
     g_get_current_time(&timeout);
     timeout.tv_sec+= 1;
     task= static_cast<GRTTaskBase*>(g_async_queue_timed_pop(task_queue, &timeout));
+#endif
     if (!task)
       continue;
 
-    self->_busy= true;
-    DPRINT("%s", std::string("worker: got task '"+task->name()+"' handling...").c_str());
+    base::atomic_int_inc(&self->_busy);
+    log_debug3("GRT dispatcher, running task %s", task->name().c_str());
 
     if (dynamic_cast<NULLTask*>(task) != 0) // a NULL task terminates the thread
     {
       DPRINT("worker: termination task received, closing...");
       task->finished(grt::ValueRef());
       task->release();
+      base::atomic_int_dec_and_test_if_zero(&self->_busy);
       break;
     }
 
@@ -509,6 +461,7 @@ gpointer GRTDispatcher::worker_thread(gpointer data)
     {
       DPRINT("%s", std::string("worker: task '"+task->name()+"' was cancelled.").c_str());
       task->release();
+      base::atomic_int_dec_and_test_if_zero(&self->_busy);
       continue;
     }
     
@@ -522,8 +475,9 @@ gpointer GRTDispatcher::worker_thread(gpointer data)
 
     if (task->get_error())
     {
-      DPRINT("%s", std::string(("worker: task '"+task->name()+"' has failed with error:.")+task->get_error()->what()).c_str());
+      log_error("%s\n", std::string(("worker: task '"+task->name()+"' has failed with error:.")+task->get_error()->what()).c_str());
       task->release();
+      base::atomic_int_dec_and_test_if_zero(&self->_busy);
       continue;
     }
 
@@ -534,6 +488,7 @@ gpointer GRTDispatcher::worker_thread(gpointer data)
     // cleanup
     task->release();
 
+    base::atomic_int_dec_and_test_if_zero(&self->_busy);
     DPRINT("worker: task finished.");
   }
 
@@ -542,9 +497,10 @@ gpointer GRTDispatcher::worker_thread(gpointer data)
   g_async_queue_unref(task_queue);
   g_async_queue_unref(callback_queue);
 
-  self->_worker_running= false;
+//  self->_worker_running= false;
+  self->_w_runing.post();
 
-  DPRINT("worker thread exiting...");
+  log_debug("worker thread exiting...\n");
 
   return NULL;
 }
@@ -552,13 +508,13 @@ gpointer GRTDispatcher::worker_thread(gpointer data)
 
 void GRTDispatcher::execute_now(GRTTaskBase *task)
 {
-  _busy= true;
+  base::atomic_int_inc(&_busy);
   prepare_task(task);
   
   execute_task(task);
   
   task->release();
-  _busy= false;
+  base::atomic_int_dec_and_test_if_zero(&_busy);
 }
 
 
@@ -585,7 +541,7 @@ void GRTDispatcher::cancel_task(GRTTaskBase *task)
 
 bool GRTDispatcher::get_busy()
 {
-  return (_task_queue && g_async_queue_length(_task_queue) > 0) || _busy;
+  return (_task_queue && g_async_queue_length(_task_queue) > 0) || base::atomic_int_get(&_busy);
 }
 
 
@@ -614,7 +570,7 @@ void GRTDispatcher::flush_pending_callbacks()
     {
       // Don't run any task, but clear the queue if we are shutting down.
       // This way the NullTask can surface up in the worker thread.
-      if (!_worker_shutdown)
+      if (!_shutdown_callback)
         callback->execute();
       callback->signal();
       callback->release();  // this release <--------------------------------------------------+
@@ -663,7 +619,7 @@ void GRTDispatcher::worker_thread_init()
 
 void GRTDispatcher::worker_thread_release()
 {
-//QQQ  _grt->disable_thread_notifications();
+  mforms::Utilities::driver_shutdown();
 }
 
 void GRTDispatcher::worker_thread_iteration()
@@ -755,6 +711,12 @@ void GRTDispatcher::execute_task(GRTTaskBase *gtask)
     restore_callbacks(gtask);
     gtask->failed(*error);
   }
+  catch (...)
+  {
+    log_error("Unknown exception in grt execute_task.");
+    restore_callbacks(gtask);
+    gtask->failed(std::runtime_error("Unknown reason"));
+  }
 }
 
 
